package backend

import (
	"context"
	"kpt-pasture/config"
	"kpt-pasture/service/wechat"
	"kpt-pasture/store/kptstore"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
	"gitee.com/xuyiping_admin/pkg/di"

	"go.uber.org/dig"
)

var Module = di.Options(di.Provide(NewStore))

type Hub struct {
	dig.In
	OpsService KptService
}

type StoreEntry struct {
	dig.In

	Cfg        *config.AppConfig
	DB         *kptstore.DB
	HttpClient *wechat.ClientService
}

func NewStore(store StoreEntry) KptService {
	return &store
}

func NewStoreEntry(cfg *config.AppConfig, Db *kptstore.DB) *StoreEntry {
	return &StoreEntry{
		Cfg:        cfg,
		DB:         Db,
		HttpClient: nil,
	}
}

//go:generate mockgen -destination mock/kptservice.go -package kptservicemock kpt-pasture/module/backend KptService
type KptService interface {
	SystemService        // 系统相关操作
	PastureManageService // 牧场管理相关
	ConfigDataService    // 配置数据相关
}

//go:generate mockgen -destination mock/SystemService.go -package kptservicemock kpt-pasture/module/backend SystemService
type SystemService interface {
	// Login 系统用户相关
	Login(ctx context.Context, req *pasturePb.SearchUserRequest) (*pasturePb.SystemUserResponse, error)
	SearchSystemUserList(ctx context.Context, req *pasturePb.SearchUserRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchUserResponse, error)
	IsShowSystemUser(ctx context.Context, userId int64) error
	DeleteSystemUser(ctx context.Context, userId int64) error
	SystemUserCreateOrUpdate(ctx context.Context, req *pasturePb.SearchUserRequest) error
	ResetPasswordSystemUser(ctx context.Context, req *pasturePb.ResetUserPasswordRequest) error
	SystemUserRole(ctx context.Context, userId int64) (*pasturePb.SystemUserRoleResponse, error)
	SystemUserRoleSave(ctx context.Context, req *pasturePb.SystemUserRoleRequest) error

	// GetSystemUserMenu 当前登录用户菜单权限
	GetSystemUserMenu(ctx context.Context) (*pasturePb.SystemUserMenuTreeResponse, error)

	GetCurrentUserName(ctx context.Context) (string, error)

	// SearchSystemRoleList 系统角色相关
	SearchSystemRoleList(ctx context.Context, req *pasturePb.SearchRoleRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchRoleResponse, error)
	DeleteSystemRole(ctx context.Context, roleId int64) error
	IsShowSystemRole(ctx context.Context, roleId int64) error
	SystemRoleCreateOrUpdate(ctx context.Context, req *pasturePb.SearchRoleRequest) error
	GetRoleMenuList(ctx context.Context, roleId int64) (*pasturePb.SystemRoleMenuResponse, error)
	RoleMenuSave(ctx context.Context, res *pasturePb.SystemRoleMenuRequest) error
	SystemRoleList(ctx context.Context) (*pasturePb.GetRoleListResponse, error)

	// SearchSystemMenuList 系统菜单权限
	SearchSystemMenuList(ctx context.Context, req *pasturePb.SearchMenuRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchMenuResponse, error)
	DeleteSystemMenu(ctx context.Context, menuId int64) error
	CreateOrUpdateSystemMenu(ctx context.Context, req *pasturePb.SearchMenuRequest) error
	SystemMenuTree(ctx context.Context) (*pasturePb.SystemMenuTreeResponse, error)

	// SearchSystemDeptList 部门列表
	SearchSystemDeptList(ctx context.Context, req *pasturePb.SearchDeptRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchDeptResponse, error)
	SystemDepDelete(ctx context.Context, id int64) error
	SystemDeptCreateOrUpdate(ctx context.Context, req *pasturePb.SearchDeptRequest) error
}

//go:generate mockgen -destination mock/PastureManageService.go -package kptservicemock kpt-pasture/module/backend PastureManageService
type PastureManageService interface {
	SearchBarnList(ctx context.Context, req *pasturePb.SearchBarnRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchBarnResponse, error)
	CreateOrUpdateBarn(ctx context.Context, req *pasturePb.SearchBarnList) error
}

//go:generate mockgen -destination mock/ConfigDataService.go -package kptservicemock kpt-pasture/module/backend ConfigDataService
type ConfigDataService interface {
	BarnTypeList(ctx context.Context) (*pasturePb.BarnTypeListResponse, error)
}
